package filters;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;
/**
 * This object holds the parameters which define a filter in a Venn diagram.
 * It maintains a EventListener list so as to be able to alert from changes on
 * the parameters. In this way the program need not check all parameters
 * every time.
 * 
 * @author Etienne Fort
 */
public class FilterParam {
	/**
	 * Specifies the behaviour of parameters FilterParam regarding events
	 * broadcasting. Comment for <code>alertListeners</code>
	 */
	public static boolean ALERT_LISTENERS = true;
	private int columnIndex;
	private String columnText = "";
	private EventListenerList listenerList = new EventListenerList();
	/**
	 * Threshold control of filtering operation. Varies from 0.0 to 1.0
	 * Comment for <code>threshold</code>
	 */
	private double threshold;
	/**
	 * Creates a new FilterParam
	 * 
	 * @param i
	 *            the column index of the column to be filtered with.
	 * @param string
	 *            the column title.
	 * @param d
	 *            the threshold below which lines are not displayed. Is supposed
	 *            to vary from 0.0 to 1.0
	 */
	public FilterParam(int i, String string, double d) {
		columnIndex = i;
		columnText = string;
		threshold = d;
	}
	/**
	 * Adds a listener of changing event, sent when the param changes.
	 * @param l 
	 * 				the listener to add.
	 */
	public void addChangeListener(ChangeListener l) {
		listenerList.add(ChangeListener.class, l);
	}
	/**
	  * Notifies all listeners that have registered interest for notification on
	  * this event type. The event instance is lazily created using the
	  * parameters passed into the fire method.
	  */
	public void fireStateChanged() {
		ChangeEvent changeEvent = null;
		// Guaranteed to return a non-null array
		Object[] listeners = listenerList.getListenerList();
		// Process the listeners last to first, notifying
		// those that are interested in this event
		if (ALERT_LISTENERS)
			for (int i = listeners.length - 2; i >= 0; i -= 2) {
				if (listeners[i] == ChangeListener.class) {
					// Lazily create the event:
					if (changeEvent == null)
						changeEvent = new ChangeEvent(this);
					((ChangeListener) listeners[i + 1]).stateChanged(changeEvent);
				}
			}
	}
	/**
	 * @return the columnIndex.
	 */
	public int getColumnIndex() {
		return columnIndex;
	}
	/**
	 * @return the columnText.
	 */
	public String getColumnText() {
		return columnText;
	}
	/**
	 * @return the threshold.
	 */
	public double getThreshold() {
		return threshold;
	}
	/**
	 * Removes a listener from the list of event listeners.
	 * @param l 
	 * 				the listener to remove.
	 */
	public void removeChangeListener(ChangeListener l) {
		listenerList.remove(ChangeListener.class, l);
	}
	/**
	 * @param columnIndex
	 *            The columnIndex to set.
	 */
	public void setColumnIndex(int columnIndex) {
		this.columnIndex = columnIndex;
		fireStateChanged();
	}
	/**
	 * @param columnText
	 *            The columnText to set.
	 */
	public void setColumnText(String columnText) {
		this.columnText = columnText;
		fireStateChanged();
	}
	/**
	 * @param threshold
	 *            The threshold to set.
	 */
	public void setThreshold(double threshold) {
		this.threshold = threshold;
		fireStateChanged();
	}
}
