package filters;
import java.util.Arrays;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.TableModel;

import tableoperations.TableFilter;

/**
 * A Filter for TableModels, given an array of parameters FilterParam. This
 * Filter listens to its parameters so as to be up to date at every time it is
 * needed. However calling reallocateIndexes may be used to refresh the contents
 * of the table. fireTableDataChanged alerts all the listeners that the table has
 * changed. Usually a call to reallocateIndexes should be followed by a call to
 * fireTableDataChanged.
 * 
 * @author Etienne Fort
 * @see filters.FilterParam
 */
public class TableParamsFilter extends TableFilter {
	private Double[] maxValues;
	private Double[] minValues;
	private FilterParam[] params;
	/**
	 * Creates a new TableParamsFilter given the original table and a set of
	 * params. The table registers itself as a listener to the params.
	 * 
	 * @param model
	 *            the original table
	 * @param params
	 *            the set of params taken into account
	 */
	public TableParamsFilter(TableModel model, FilterParam[] params) {
		setParams(params);
		for (int i = 0; i < params.length; i++) {
			params[i].addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent arg0) {
					reallocateIndexes();
					fireTableDataChanged();
				}
			});
		};
		setModel(model);
	}
	/**
	 * Filters Double values, given a FilterParam. Doubles are filtered
	 * relatively to the original table max values and min values in the fixed
	 * column.
	 * 
	 * @see tableoperations.TableFilter#filterFunction(int)
	 */
	public boolean filterFunction(int row) {
		for (int i = 0; i < params.length; i++) {
			Object cell = model.getValueAt(row, params[i].getColumnIndex());
			if (!(cell instanceof Double))
				return false;
			Double value = (Double) cell;
			if (maxValues[i] == null)
				return false;
			if (minValues[i] == null)
				return false;
			if (value.doubleValue() < params[i].getThreshold()
					* (maxValues[i].doubleValue() - minValues[i].doubleValue())
					+ minValues[i].doubleValue())
				return false;
		}
		return true;
	}
	/**
	 * Besides computing the mapping Arraylist, computes first the max
	 * and min values in the filtered column.
	 * 
	 * @see tableoperations.TableFilter#reallocateIndexes()
	 */
	public void reallocateIndexes() {
		Arrays.fill(minValues, null);
		Arrays.fill(maxValues, null);
		for (int i = 0; i < model.getRowCount(); i++)
			for (int j = 0; j < params.length; j++) {
				Object token = model.getValueAt(i, params[j].getColumnIndex());
				if (token instanceof Double) {
					Double value = (Double) token;
					if (!value.isInfinite() && !value.isNaN()) {
						if ((maxValues[j] == null)
								|| (value.compareTo(maxValues[j]) > 0))
							maxValues[j] = value;
						if ((minValues[j] == null)
								|| (value.compareTo(minValues[j]) < 0))
							minValues[j] = value;
					}
				}
			}
		super.reallocateIndexes();
	}
	/**
	 * Defines the params for the filtering operation.
	 * 
	 * @param params
	 *            The params to set.
	 */
	public void setParams(FilterParam[] params) {
		this.params = params;
		minValues = new Double[params.length];
		maxValues = new Double[params.length];
		
	}
}
