package graphs;
import filters.FilterParam;
import filters.TableParamsFilter;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;

import javax.swing.table.TableModel;
/**
 * Subclass of GraphPanel for two graphs.
 * 
 * @see graphs.GraphPanel
 * 
 * @author Thomas Hugel, Julien Robert
 */
public class GraphPanel2 extends GraphPanel {
	private static final Color COLOR1 = Color.RED;
	private static final Color COLOR2 = Color.BLUE;
	private Point[] coordsLeft;
	private Point[] coordsRight;
	protected Point[][] filteringResults;
	/**
	 * Constructs both graphs dynamically in terms of input thresholds.
	 * 
	 * @param params
	 *            	the input parameters of the filtering
	 * @param model
	 *            	the support of the filtering
	 * @param vRes
	 * 				the vertical resolution
	 * @param thrRange1
	 * 				the range of thresholds (min - max) for the first filtering
	 * @param thrRange2
	 *				the range of thresholds (min - max) for the second filtering
	*/
	public GraphPanel2(FilterParam[] params, TableModel model, int vResolution,
			double[] thrRange1, double[] thrRange2) {
		super(model, vResolution);
		coordsLeft = new Point[2 * verticalResolution];
		coordsRight = new Point[2 * verticalResolution];
		FilterParam[] paramsArray1 = new FilterParam[1];
		paramsArray1[0] = params[0];
		TableParamsFilter filteredTable1 = new TableParamsFilter(model,
				paramsArray1);
		FilterParam[] paramsArray2 = new FilterParam[1];
		paramsArray2[0] = params[1];
		TableParamsFilter filteredTable2 = new TableParamsFilter(model,
				paramsArray2);
		filteringResults = calculateFilteringResults(params, filteredTable1,
				filteredTable2, thrRange1, thrRange2);
		widthNb = 2* Math.max(filteringResults[1][verticalResolution - 1].x,
					 filteringResults[2][verticalResolution - 1].x);
		}
	/**
	 * Commands the starting of the filterings and tabulates the results. There are
	 * three areas: one for each of both single filters (results[1] and [2]) and one for
	 * the intersection of these filters (results[0]).
	 * 
	 * @param params
	 *            the input parameters for the filtering
	 * @param filteredTable1
	 *            the first filtered table
	 * @param filteredTable2
	 *            the second filtered table
	 * @param thrRange1
	 *            the range of thresholds for the first filtering
	 * @param thrRange2
	 *            the range of thresholds for the second filtering
	 * @return the results of the filtering for the different threshold values.
	 */
	public Point[][] calculateFilteringResults(FilterParam[] params,
			TableParamsFilter filteredTable1, TableParamsFilter filteredTable2,
			double[] thrRange1, double[] thrRange2) {
		Point[][] results = new Point[3][verticalResolution];
		//		intersection of both filters
		TableParamsFilter intersectionTable = new TableParamsFilter(
				originalTable, params);
		boolean OLD_VAL = FilterParam.ALERT_LISTENERS;
		FilterParam.ALERT_LISTENERS = false;
		for (int i = 0; i < verticalResolution; i++) {
			params[0].setThreshold(ordToThreshold(i, thrRange1));
			params[1].setThreshold(ordToThreshold(i, thrRange2));
			filteredTable1.reallocateIndexes();
			filteredTable2.reallocateIndexes();
			intersectionTable.reallocateIndexes();
			results[0][i] = new Point(intersectionTable.getRowCount(), i);
			results[1][i] = new Point(filteredTable1.getRowCount(), i);
			results[2][i] = new Point(filteredTable2.getRowCount(), i);
		}
		FilterParam.ALERT_LISTENERS = OLD_VAL;
		return (results);
	}
	/**
	 * @param singleResults
	 *            results of a single filtering
	 * @param doubleResults
	 *            results of the intersection between two filters
	 * @return the coordinates of the left graph when there are two graphs
	 * 
	 * This is a representation of singleResults translated according to
	 * doubleResults.
	 */
	public Point[] computeCoordsLeft(Point[] singleResults,
			Point[] doubleResults) {
		for (int i = 0; i < verticalResolution; i++) {
			coordsLeft[i + verticalResolution] = new Point(
					(int) (middleAbscissa + (doubleResults[i].x) * scaleW / 2.),
					(int) ((singleResults[i].y) * scaleH));
			coordsLeft[verticalResolution - i - 1] = new Point((coordsLeft[i
					+ verticalResolution].x)
					- (int) (singleResults[i].x * scaleW), coordsLeft[i
					+ verticalResolution].y);
		}
		return (coordsLeft);
	}
	/**
	 * @see GraphPanel2#computeCoordsLeft(Point[], Point[])
	 */
	public Point[] computeCoordsRight(Point[] singleResults,
			Point[] doubleResults) {
		for (int i = 0; i < verticalResolution; i++) {
			coordsRight[verticalResolution - i - 1] = new Point(
					(int) (middleAbscissa - (doubleResults[i].x) * scaleW / 2.),
					(int) ((singleResults[i].y) * scaleH));
			coordsRight[i + verticalResolution] = new Point(
					(coordsRight[verticalResolution - i - 1].x)
							+ (int) (singleResults[i].x * scaleW),
					coordsRight[verticalResolution - i - 1].y);
		}
		return (coordsRight);
	}
	/**
	 * @see java.awt.Component#paint(java.awt.Graphics)
	 */
	public void paint(Graphics graph) {
		super.paint(graph);
		Point[] coords1 = computeCoordsLeft(filteringResults[1], filteringResults[0]);
		drawCurve(graph, COLOR1, coords1);
		graph.drawString("seuil 1", 1, fontSize);
		Point[] coords2 = computeCoordsRight(filteringResults[2], filteringResults[0]);
		drawCurve(graph, COLOR2, coords2);
		graph.drawString("seuil 2", (int) getWidth() - 45, fontSize);
	}
	
	/**
	 * @see java.awt.print.Printable#print(java.awt.Graphics, java.awt.print.PageFormat, int)
	 */
	public int print(Graphics g, PageFormat pageFormat, int nbPages) throws PrinterException {
		if (nbPages>=1) return NO_SUCH_PAGE;
		else 
			{
			double X= pageFormat.getImageableX();
			double Y= pageFormat.getImageableY();
			double width = pageFormat.getImageableWidth();
			double height = pageFormat.getImageableHeight();
			
			middleAbscissa = new Double(width/2).intValue() + new Double(X).intValue();
			int fontSize = (int) (g.getFont().getSize2D()) + 1;
			scaleH = height / (double) verticalResolution;
			scaleW = width / (double) widthNb;
			 g.drawString("nb gnes", middleAbscissa - 10, (int) height - 3);
			
			Point[] coords1 = computeCoordsLeft(filteringResults[1], filteringResults[0]);
			drawCurve(g, COLOR1, coords1);
			g.drawString("seuil 1", 1+(int)X, fontSize + (int)Y);
			Point[] coords2 = computeCoordsRight(filteringResults[2], filteringResults[0]);
			drawCurve(g, COLOR2, coords2);
			g.drawString("seuil 2", (int) width - 50 + (int) X, fontSize + (int)Y);

			return PAGE_EXISTS;
	
			}
	}



}