package tableoperations;
import java.util.ArrayList;

import javax.swing.event.TableModelEvent;
import javax.swing.table.TableModel;
/**
 * A filter for TableModels. The filter has a model (conforming to TableModel)
 * and implements itself TableModel. TableFilter neither stores nor copies the
 * data in the TableModel, instead it maintains an arraylist of integers which
 * keeps the current rows that are not to be filtered. When the model changes
 * it notifies the filter that something has changed (e.g. "rowsAdded") so that
 * its internal array of integers can be reallocated. As requests are made of
 * the filter (like getValueAt(row, col) it redirects them to its model via the
 * mapping arraylist. In this way the TableFilter appears to hold another copy of
 * the table with the filtered rows. The class TableFilter is abstract so it
 * has to be subclassed in order to specify the filter function.
 * 
 * @see TableSorter
 * 
 * @author Etienne Fort
 */
public abstract class TableFilter extends TableMap {
	int rowCount;
	ArrayList indexes;
	/**
	 * Creates a new empty <code>TableFilter</code>.
	 */
	public TableFilter() {
		indexes = new ArrayList();
	}
	/**
	 * Creates a new <code>TableFilter</code> on the <code>TableModel</code> model.
	 * 
	 * @param model
	 *            the <code>TableModel</code> which is filtered.
	 */
	public TableFilter(TableModel model) {
		indexes = new ArrayList();
		setModel(model);
	}
	public void setModel(TableModel model) {
		super.setModel(model);
		reallocateIndexes();
		fireTableDataChanged();
	}
	/**
	 * Allows to filter the table. This function is supposed to return true if
	 * the line belongs to the table, false otherwise
	 * 
	 * @param row
	 *            index of the row to check.
	 * @return false if the line must be ignored, true otherwise
	 */
	public abstract boolean filterFunction(int row);
	/**
	 * Computes the mapping arraylist of indexes.
	 */
	public void reallocateIndexes() {
		rowCount = model.getRowCount();
		// Set up a new arraylist of indexes with the right number of elements
		// for the new data model.
		indexes.clear();
		for (int i = 0; i < rowCount; i++)
			if (filterFunction(i))
				indexes.add(new Integer(i));
	}
	/**
	 * Called when the model has changed. Then recomputes the mapping arraylist
	 * and forwards the tablechanged event to the listeners of this TableFilter.
	 * 
	 * @see javax.swing.event.TableModelListener#tableChanged(javax.swing.event.TableModelEvent)
	 */
	public void tableChanged(TableModelEvent e) {
		//System.out.println("Filter: tableChanged");
		reallocateIndexes();
		super.tableChanged(e);
	}
	public void checkModel() {
		if (rowCount != model.getRowCount()) {
			System.err.println("Filter not informed of a change in model.");
		}
	}
	// The mapping only affects the contents of the data rows.
	// Pass all requests to these rows through the mapping arraylist:
	// "indexes".
	public int getRowCount() {
		return indexes.size();
	}
	/**
	 * @see javax.swing.table.TableModel#isCellEditable(int, int)
	 */
	public boolean isCellEditable(int row, int column) {
		return super.isCellEditable(((Integer) indexes.get(row)).intValue(),column);
	}
	/**
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int aRow, int aColumn) {
		checkModel();
		return model.getValueAt(((Integer) indexes.get(aRow)).intValue(),aColumn);
	}
	/**
	 * @see javax.swing.table.TableModel#setValueAt(java.lang.Object, int, int)
	 */
	public void setValueAt(Object aValue, int aRow, int aColumn) {
		checkModel();
		model.setValueAt(aValue, ((Integer) indexes.get(aRow)).intValue(),aColumn);
	}
}
