package tableoperations;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

/**
 * In a chain of data manipulators some behaviour is common. TableMap
 * provides most of this behaviour and can be subclassed by filters
 * that only need to override a handful of specific methods. TableMap
 * implements TableModel by routing all requests to its model, and
 * TableModelListener by routing all events to its listeners. Inserting
 * a TableMap which has not been subclassed into a chain of table filters
 * should have no effect.
 *
 * @author Philip Milne, Julien Robert for writeToFileAsCsv
 */
public class TableMap extends AbstractTableModel
implements TableModelListener {
	protected TableModel model;
	
	public TableModel getModel() {
		return model;
	}
	
	public void setModel(TableModel model) {
		this.model = model;
		model.addTableModelListener(this);
	}
	
	// By default, implement TableModel by forwarding all messages
	// to the model.
	
	public Object getValueAt(int aRow, int aColumn) {
		return model.getValueAt(aRow, aColumn);
	}
	
	public void setValueAt(Object aValue, int aRow, int aColumn) {
		model.setValueAt(aValue, aRow, aColumn);
	}
	
	public int getRowCount() {
		return (model == null) ? 0 : model.getRowCount();
	}
	
	public int getColumnCount() {
		return (model == null) ? 0 : model.getColumnCount();
	}
	
	public String getColumnName(int aColumn) {
		return model.getColumnName(aColumn);
	}
	
	public Class getColumnClass(int aColumn) {
		return model.getColumnClass(aColumn);
	}
	
	public boolean isCellEditable(int row, int column) {
		return model.isCellEditable(row, column);
	}
//	
//	Implementation of the TableModelListener interface,
//	

	// By default forward all events to all the listeners.
	public void tableChanged(TableModelEvent e) {
		fireTableChanged(e);
	}
	
	/**
	 * Writes the table contents in a table file using CSV (comma separated
	 * values) format. 
	 * 
	 * @author Julien Robert
	 * 
	 * @param fileName
	 *            	name of the written file
	 * @param fileHasHeaders
	 * 
	 * @throws IOException
	 */
	public void writeToFileAsCsv(String fileName, boolean fileHasHeaders) throws IOException {
		int nbColonnes, nbLignes, k;
		String cse;
		nbLignes = this.getRowCount();
		nbColonnes = this.getColumnCount();
		OutputStreamWriter opsw = new FileWriter(fileName);
		BufferedWriter bw = new BufferedWriter(opsw);
		for (int i = -1; i < nbLignes; i++) {
			for (int j = 0; j < nbColonnes; j++) {
				Object value;
				if (i == -1)
					value = fileHasHeaders ? getColumnName(j) : null;
				else
					value = getValueAt(i, j);
				if (value != null)
					if (value instanceof String) {
						bw.write('"');
						cse = value.toString();
						for (k = 0; k < cse.length(); k++) {
							if (cse.charAt(k) == '"')
								bw.write('"');
							bw.write(cse.charAt(k));
						}
						bw.write('"');
					} else
						bw.write(value.toString(), 0, value.toString().length());
				if (j < nbColonnes - 1)
					bw.write(';');
			}
			bw.newLine();
			bw.flush();
		}
		bw.close();
	}
}
