package venndiagrams;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.MouseInputAdapter;
import javax.swing.table.TableModel;

import windows.MainWindow;
import filters.FilterParam;
import filters.TableParamsFilter;
/**
 * Graphic Venn diagram (or potatoes)
 * 
 * @author Julien Robert, Thomas Hugel, Etienne Fort
 *  
 */
public class Venn3DiagramPanel extends VennPanel {
	/**
	 * A potato is a circle containing some informations relative to the filterings.
	 * The 3 potatoes don't have the same role. That is the reason why they have a number. 
	 * The other fields are graphic.
	 */
	private class Potato {
		private int num;
		private int x, y, radius, textX, textY;
		private double alpha, beta;
		public Potato(int num) {
			this.num = num;
		}
		/**
		 * Gets the result of the filtering (number of selected lines)
		 * and sets the proper color consequently
		 * 
		 * @param g
		 * 				the graphic area
		 * @param index
		 * 				the column index
		 * @return the result of the filtering
		 */
		private String getNbLinesAndSetColor(Graphics g, int index){
			int result = filteredTables[index].getRowCount();
			g.setColor(getThresholdColor((double)result/(double)originalTable.getRowCount()));
			return Integer.toString(result);
		}
		/**
		 * Decides whether the point (x, y) is in the circle 
		 */
		public boolean isInPotato(int x, int y) {
			int dx = x - this.x;
			int dy = y - this.y;
			return (dx * dx + dy * dy <= (radius * radius));
		}
		/**
		 * Draws the num-th potato with the results inside.
		 * 
		 * @see javax.swing.JComponent#paint(java.awt.Graphics)
		 */
		public void paint(Graphics g) {
			g.setColor(Color.BLACK);
			g.drawArc(x - radius, y - radius, radius * 2, radius * 2, 0, 360);
			g.drawString(filterParams[num].getColumnText(), textX, textY);
			if (textX < x)
				g.drawLine((int) (x - radius / beta), (int) (y - radius * alpha / beta), textX, textY);
			else
				g.drawLine((int) (x + radius / beta), (int) (y + radius * alpha / beta), textX, textY);	
			
			if (num == 0) {
				g.drawString(getNbLinesAndSetColor(g, 0), (int)(x - radius/2.), y);
				} 
			if (num == 1) {		
				g.drawString(getNbLinesAndSetColor(g, 1), (int)(x + radius/4.), y);
				g.drawString(getNbLinesAndSetColor(g, 2), (int)(x - radius*2/3.), (int)(y - radius/4.));
			}
			if (num == 2) {
				g.drawString(getNbLinesAndSetColor(g, 3), (int)(x - radius/6.), (int)(y + radius/4.));
				g.drawString(getNbLinesAndSetColor(g, 4), (int)(x - radius*3/4.), (int)(y - radius/4.));
				g.drawString(getNbLinesAndSetColor(g, 5), (int)(x + radius/3.), (int)(y - radius/4.));
				g.drawString(getNbLinesAndSetColor(g, 6), (int)(x - radius/6.), (int)(y - radius*2/3.));				
			}
		}
		/**
		 * Sets the graphic parameters
		 * @param x,y
		 * 				the coordinates of the potato center
		 * @param radius
		 * 				the radius of the potato
		 * @param textx,texty
		 * 				the coordinates of the label
		 */
		public void setGraphicParameters(int x, int y, int radius, int textx, int texty) {
			this.radius = radius;
			this.x = x;
			this.y = y;
			this.textX = textx;
			this.textY = texty;
			alpha = (double) (textY - y) / (double) (textX - x);
			beta = Math.sqrt(alpha * alpha + 1);
		}
	}
	protected TableParamsFilter[] filteredTables;
	protected Potato thePotatoes[];
	private String[] titles;
	private Vector[] subSetsParams;

	/**
	 * Creates the window containing the potatoes and the MouseListener, which
	 * works so: as soon as the user clicks on an area in a potato, each potato
	 * is asked whether it has been clicked upon; in that case res is increased
	 * of 2**(potato number); so it is possible to know where the user clicked
	 * (thanks to a clever calculation in basis 1-2-4) 
	 * and to open a window displaying the involved genes.
	 * 
	 *@see the superclass VennPanel
	 */
	public Venn3DiagramPanel(TableModel table, FilterParam[] params) {
		super(table, params);
		thePotatoes = new Potato[filterParams.length];
		for (int i = 0; i < filterParams.length; i++) {
			thePotatoes[i] = new Potato(i);
			filterParams[i].addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent arg0) {
					repaint();
				}
			});
		}
		filteredTables = new TableParamsFilter[7];
		subSetsParams = subSets(params);
		for (int i = 0; i < subSetsParams.length; i++) {
			FilterParam[] newParams = 
				(FilterParam[]) subSetsParams[i].toArray(new FilterParam[subSetsParams[i].size()]);
			filteredTables[i] = new TableParamsFilter(originalTable, newParams);
		}
		//Here the way subsets returns elements in a specific order is very important, see mouseclicked
		addMouseListener(new MouseInputAdapter() {
			/* (non-Javadoc)
			 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
			 */
			public void mouseClicked(MouseEvent e) {
				int x = e.getX();
				int y = e.getY();
				int result = 0;
				for (int i = 0; i < filterParams.length; i++) {
					if (thePotatoes[i].isInPotato(x, y))
						result = result + (i == 0 ? 1 : (i == 1 ? 2 : 4));
					//Here the order of elements returned by subsets is very important
				}
				FilterParam[] newParams = 
					(FilterParam[]) subSetsParams[result-1].toArray(new FilterParam[subSetsParams[result-1].size()]);
				MainWindow.showNewTableWindow("", filteredTables[result-1], newParams);
			}
		});
		addComponentListener(new ComponentAdapter() {
			/* (non-Javadoc)
			 * @see java.awt.event.ComponentAdapter#componentResized(java.awt.event.ComponentEvent)
			 */
			public void componentResized(ComponentEvent arg0) {
				int fontSize = (int)(getFont().getSize2D()) + 1;
				int minWidthHeight = Math.min(getHeight() - 2 * fontSize, getWidth());
				int radius = minWidthHeight / 3;
				if (thePotatoes.length >= 1)
					thePotatoes[0].setGraphicParameters(radius, radius
							+ fontSize, radius, 0, fontSize);
				if (thePotatoes.length >= 2)
					thePotatoes[1].setGraphicParameters(2 * radius, radius
							+ fontSize, radius, getWidth() - 70, fontSize);
				if (thePotatoes.length >= 3)
					thePotatoes[2].setGraphicParameters(minWidthHeight / 2, 2
							* radius + fontSize, radius, 0, getHeight());
			}
		});
		setPreferredSize(new Dimension(300,300));
	}
	
	/**
	 * @see java.awt.Component#paint(java.awt.Graphics)
	 */
	public void paint(Graphics g) {
		super.paint(g);
		for (int i = 0; i < thePotatoes.length; i++) {
			thePotatoes[i].paint(g);
		}
	}
}