package windows;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.TableModel;

import tableoperations.TableFile;
import tableoperations.TableSorter;

import computecolumn.ComputeColumnDialog;
import computecolumn.TableAddFunctionColumn;

import filters.FilterParam;
/**
 * The MainWindow holds a JTable table and displays it on the whole viewport
 * area of a JScrollPane object. Tables can be loaded or saved through
 * FileChooser. The Menu JMenu allows to launch different computing functions on
 * the table, including sorting, filtering and adding new computed columns. 
 * It aims at facilitating visual understanding of DNA analysis tables, by
 * providing dynamic tools to do multi-filtering operations.
 * @author Etienne Fort
 */
public class MainWindow extends JFrame implements ActionListener {
	static final String stClose = "Fermer";
	static final String stFile = "Fichier";
	static final String stGraphs = "Graphiques";
	static final String stNewColumn = "Calculer colonne";
	static final String stOpen = "Ouvrir";
	static final String stTools = "Outils";
	static final String stPotatoes = "Patates";
	static final String stQuit = "Quitter";
	static final String stSave = "Enregistrer";
	static final String stSaveAs = "Enregistrer sous ...";
	static final String stTitle = "OpenLLG";
	/**
	 * Creates the GUI and shows it. For thread safety, this method should be
	 * invoked from the event-dispatching thread.
	 */
	private static void createAndShowGUI() {
		//Makes sure we have nice window decorations.
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);
		//Creates and sets up the window.
		JFrame frame = new MainWindow();
		//Displays the window.
		frame.setTitle(stTitle);
		Image icon = frame.getToolkit().getImage("icone.jpg");
		frame.setIconImage(icon);
		frame.pack();
		frame.setLocationRelativeTo(null);
		frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		frame.setVisible(true);
	}
	/**
	 * Comment is needless.
	 */
	public static void main(String args[]) {
		//Schedules a job for the event-dispatching thread
		//creating and showing this application's GUI.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				createAndShowGUI();
			}
		});
	}
	/**
	 * Creates a new window from the table TableModel.
	 * 
	 * @param title
	 *            the title of the window.
	 * @param table
	 *            the table which is to be displayed through the window.
	 */
	public static void showNewTableWindow(String title, TableModel table, FilterParam[] params) {
		//Creates and sets up the window.
		JFrame frame = new MainWindow(table, params);
		//Displays the window.
		if (params == null)
			frame.setTitle(title);
		frame.pack();
		frame.setLocationRelativeTo(null);
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		frame.setVisible(true);
	}
	protected String fileName = "";
	protected TableModel dataTable;
	protected JTable tableDisplay;
	private TableSorter tableSorted;
	
	/**
	 * Creates a new MainWindow with an empty table.
	 */
	public MainWindow() {
		tableDisplay = new JTable();
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent arg0) {
				dispose();
			}
		});
		tableDisplay.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		setContentPane(new JScrollPane(tableDisplay,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));

		createMenu();
	};
	/**
	 * @return a frame title corresponding to the parameters
	 */
	protected String computeParamsTitle(String init, FilterParam[] params, String oldTitle) {
		String title = init;
		if (params != null)
			for (int i = 0; i < params.length; i++) {
				if (i == 0)
					title = title + params[i].getColumnText();
				else
					title = title + " & " + params[i].getColumnText();
			}
		else
			title = oldTitle;
		return title;
	}
	/**
	 * Creates a new MainWindow with the given table.
	 * 
	 * @param table
	 *            the table to be displayed.
	 * @param params
	 *            the params of filtering (if any) in order to display the righ title.
	 */
	public MainWindow(TableModel table, final FilterParam[] params) {
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		if (params != null)
			for (int i = 0; i < params.length; i++) {
				params[i].addChangeListener(new ChangeListener() {
					public void stateChanged(ChangeEvent arg0) {
						if (fileName == "")
							setTitle(computeParamsTitle("Filtrage sur ", params, getTitle()));
						else
							setTitle(fileName);
					}
				});
			}
		setTitle(computeParamsTitle("Filtrage sur ", params, getTitle()));
		dataTable = table;
		tableSorted = new TableSorter(dataTable);
		tableDisplay = new JTable(tableSorted);
		tableSorted.addMouseListenerToHeaderInTable(tableDisplay);
		tableDisplay.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		setContentPane(new JScrollPane(tableDisplay,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
		createMenu();
	}
	/**
	 * Dispatches action events received from the menu by calling the appropriate
	 * functions.
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		String A = e.getActionCommand();
		if (A == stClose)
			dispatchEvent(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));
		else if (A == stQuit) {
			System.exit(0);
		} else if (A == stOpen) {
			if (openFileDialog(stOpen))
				fileOpen();
		} else if (dataTable != null) {
			if (A == stSave) {
				fileSave();
			} else if (A == stSaveAs) {
				if (openFileDialog(stSave))
					fileSave();
			} else if (A == stNewColumn) {
				openNewColumnDialog();
			} else if (A == stPotatoes) {
				openDiagramWindow();
			} else if (A == stGraphs) {
				openGraphWindow();
			}
		} else
			System.out.print(A+"\n");
	}
	protected void createMenu() {
		JMenuBar menu = new JMenuBar();
		JMenu fichier = new JMenu(stFile);
		JMenu outils = new JMenu(stTools);
		fichier.add(stOpen).addActionListener(this);
		fichier.add(stSave).addActionListener(this);
		fichier.add(stSaveAs).addActionListener(this);
		fichier.add(new JSeparator());
		fichier.add(stClose).addActionListener(this);
		fichier.add(stQuit).addActionListener(this);
		outils.add(stNewColumn).addActionListener(this);
		outils.add(stPotatoes).addActionListener(this);
		outils.add(stGraphs).addActionListener(this);
		menu.add(fichier);
		menu.add(outils);
		setJMenuBar(menu);
	}
	/**
	 * Opens the file at CSV format chosen in a prompt dialog box and displays
	 * its contents in a table.
	 */
	public void fileOpen() {
		try {
			TableModel tableau = new TableFile(fileName);
			dataTable = tableau;
			tableSorted = new TableSorter(dataTable);
			tableDisplay.setModel(tableSorted);
			tableSorted.addMouseListenerToHeaderInTable(tableDisplay);
			pack();
			setTitle(fileName);
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, "Ouverture du fichier",
					"L'ouverture du fichier a chou",
					JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
		}
	}
	/**
	 * Saves the table in the file fileName at CSV format.
	 */
	public void fileSave() {
		try {
			if ((fileName != "") || openFileDialog(stSave)) {
				tableSorted.writeToFileAsCsv(fileName, true);
				setTitle(fileName);
			}
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, "Enregistrement du fichier",
					"L'enregistrement du fichier a chou",
					JOptionPane.ERROR_MESSAGE);
			e.printStackTrace();
		}
	}
	/**
	 * Opens the diagram window corresponding to the table displayed in this
	 * window.
	 */
	public void openDiagramWindow() {
		DiagramWindow.showDiagramWindow(this, "Patates pour " + getTitle(), dataTable);
	}
	
	/**
	 * Opens a dialog box to choose a filename
	 * 
	 * @return true if the file has changed, false otherwise
	 */
	public boolean openFileDialog(String approveButtonText) {
		JFileChooser chooser = new JFileChooser(fileName);
		FileFilter filtre = new FileFilter() {
		//To reject not-csv files
			public boolean accept(File f) {
				return (f.getName().endsWith("csv") | f.isDirectory());
			}
			public String getDescription() {
				return "CSV (sparateur point virgule)";
			}
		};
		chooser.setFileFilter(filtre);
		chooser.setApproveButtonText(approveButtonText);
		chooser.setDialogTitle(approveButtonText);
		int returnVal = chooser.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			fileName = chooser.getSelectedFile().getPath(); 
			return true;
		} else
			return false;
	}
	/**
	 * Opens the windows displaying the graphs.
	 */
	public void openGraphWindow() {
		GraphWindow.showGraphWindow(this, dataTable);
	}
	/**
	 * Opens a dialog box to add a new computed column.
	 */
	public void openNewColumnDialog() {
		String[] headers = new String[dataTable.getColumnCount()];
		for (int i = 0; i < headers.length; i++) {
			headers[i] = dataTable.getColumnName(i);
		}
		ComputeColumnDialog paramWindow = new ComputeColumnDialog(this, "Nouvelle colonne", headers);
		paramWindow.pack();
		paramWindow.setLocationRelativeTo(this);
		paramWindow.setVisible(true);
		if (paramWindow.getButtonClicked() == ComputeColumnDialog.stOk) {
			TableModel table = new TableAddFunctionColumn(dataTable,
					paramWindow.getColumn1(), paramWindow.getColumn2(),
					paramWindow.getFunction());
			MainWindow.showNewTableWindow("Colonne ajoute: "
					+ table.getColumnName(table.getColumnCount() - 1), table, null);
		}
	}
}
